from construct import *
from hexdump import hexdump

EDTB2 = Struct( # Working with a Byte-reversed copy of data
    Padding(4),
    "control" / Bitwise(Struct(
        Padding(3),
        "param8" / BitsInteger(8),
        "param7" / BitsInteger(8),
        "param6" / BitsInteger(8),
        "param5" / BitsInteger(8),
        "param4" / BitsInteger(8),
        "param3" / BitsInteger(13),
        "param2" / BitsInteger(13),
        "param1" / BitsInteger(13),
        "unknown" / Bit, # always '0', so far
        "id" / BitsInteger(28),
        "enabled" / Flag,
    )),
)

EDTB1 = Struct(
    #"dump" / Peek(HexDump(Bytes(24))),
    "autorev" / ByteSwapped(Bytes(18)),
    "reversed" / RestreamData(this.autorev, EDTB2), # this does not allow re-build of data :-(
)

G1ON = Struct(
    "effect1" / EDTB1,
    "effect2" / EDTB1,
    "effect3" / EDTB1,
    "effect4" / EDTB1,
    "effect5" / EDTB1,
)


data_from_bin = \
b"\x21\x00\xc0\x02\x03\x50\x00\x0a\x00\xca\x23\x28\x02\x00\x00\x00"+\
b"\x00\x00\x21\x00\x20\x8a\x02\x50\x00\x0c\x80\x01\x25\x03\x00\x00"+\
b"\x00\x00\x64\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"+\
b"\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00"+\
b"\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00"+\
b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x40\x32\x04\x00\x00\x00"+\
b"\x20\x03\x00\x00\x00\x55\x4b\x20\x47\x72\x6f\x6f\x76\x65\x20\x00"

print("From binary:")
print(hexdump(data_from_bin))
print(G1ON.parse(data_from_bin))
print()

'''
Delay:

Time:   566.(0x236) Display 1..4000 then 23 more note delays)
F.B.:   30  (0x1e)  Display 0..100
Mix:    70  (0x46)  Display 0..100
HiDMP:  10  (0x0a)  Display: 0..10
P-P:    Mono(0x00)  Display: Mono P-P 0..1
Level:  100 (0x64)  Display: 0..150
Tail:   off (0x00)  Display: off on 0..1

Z Tron:

Sense:  1     Display: -10..10, 0..19 [-1 -> 1, no 0!]
Reso:   5     Display: 0..10
Dry:    25    Display: 0..100
Level:  100   Display: 0..150

$ ./b1on_from_pedal.exe ukgroove_from_pedal.dat
Infile: ukgroove_from_pedal.dat
INPUT
f0 52 00 65 28
'''
encoded_midi = \
b"\x06\x11\x00\x00\x50\x0d\x70\x00\x08\x46\x40\x01\x00\x0c\x00\x00"+\
b"\x00\x00\x00\x00\x00\x11\x01\x00\x42\x04\x02\x28\x00\x19\x00\x0c"+\
b"\x00\x00\x00\x00\x00\x00\x00\x00\x04\x00\x21\x00\x20\x0a\x02\x50"+\
b"\x10\x00\x0c\x00\x01\x25\x03\x00\x00\x00\x00\x00\x64\x00\x01\x00"+\
b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"+\
b"\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"+\
b"\x00\x00\x00\x00\x00\x00\x00\x40\x00\x32\x04\x00\x00\x00\x20\x03"+\
b"\x00\x00\x00\x00\x55\x4b\x20\x47\x00\x72\x6f\x6f\x76\x65\x20\x00"

def unpack(packet):
    # Unpack data 7bit to 8bit, MSBs in first byte
    data = bytearray(b"")
    loop = -1
    hibits = 0

    for byte in packet:
        if loop !=-1:
            if (hibits & (2**loop)):
                data.append(128 + byte)
            else:
                data.append(byte)
            loop = loop - 1
        else:
            hibits = byte
            # do we need to acount for short sets (at end of block block)?
            loop = 6

    return(data)

data_from_midi = unpack(encoded_midi)

print("From midi:")
print(hexdump(data_from_midi))
print(G1ON.parse(data_from_midi))
print()
